#ifndef INFO_H
#define INFO_H
#include "General.h"

namespace SANC {

class Settings;

class Info
{
public:
    Info(Settings & _cfg);
    std::string version();
/*
    Process ID:
    101: e+e- -> e-e+
    102: e+e- -> mu-mu+
    103: e+e- -> ZH
    104: e+e- -> AA
    105: AA -> AA
    106: e+e- -> AZ
    107: e+e- -> tau-tau+
*/
    const int Pid;
    //--------------------------------------------------------------------------------------------------
    const double LamMinus; // Longitudinal polarization of electron beam
    const double LamPlus; // Longitudinal polarization of positron beam
    //--------------------------------------------------------------------------------------------------
    const int Scheme; // Scheme: 0 - alpha(0), 1 - Gmu, 2 - Gmu'
    const double Ecm; // Total energy of colliding particles in CM frame
    const double Ome;
    const double Alphai;
    const double AlphaMZi;
    //--------------------------------------------------------------------------------------------------
    const int Alr;
    //--------------------------------------------------------------------------------------------------
    const int Iborn;
    const int Iqed;
    const int Iew;
    const int Ilin;
    const int Ifgg;
    const int Its;
    const int Irun;
    //--------------------------------------------------------------------------------------------------
    const bool ForceOffH;
    bool ForceOffSV;
    //--------------------------------------------------------------------------------------------------
    const double Men;
    const double Mel;
    const double Mup;
    const double Mdn;
    const double Mmn;
    const double Mmo;
    const double Mch;
    const double Mst;
    const double Mtn;
    const double Mta;
    const double Mtp;
    const double Mbt;
    const double MW;
    const double MZ;
    const double MH;
    const double Wz;
    const double Ww;
    const double Wh;
    const double Wtp;
    //--------------------------------------------------------------------------------------------------
    const double CutMinCth;
    const double CutMaxCth;
    const double CutMinM34;
    const double CutMaxM34;
    const double CutMinPt;
    const double CutMaxPt;
    //--------------------------------------------------------------------------
    const long RanSeed; // Seed for RNG initialization (0 <= RanSeed <= 9E8)
    const long NevBorn; // MC statistics for Born
    const long NevVirt; // MC statistics for Virt
    const long NevHard; // MC statistics for Hard
    const long NevTot; // MC statistics for Total
    //--------------------------------------------------------------------------------------------------
    //  FOAM parameters for Born
    const int  nCellsBorn;
    const int  nSamplBorn;
    const int  nBinBorn;
    const int  EvPerBinBorn;
    const double MaxWtRejBorn;
    //--------------------------------------------------------------------------------------------------
    //  FOAM parameters for Virt
    const int  nCellsVirt;
    const int  nSamplVirt;
    const int  nBinVirt;
    const int  EvPerBinVirt;
    const double MaxWtRejVirt;
    //--------------------------------------------------------------------------------------------------
    //  FOAM parameters for Hard
    const int  nCellsHard; // Number of Cells
    const int  nSamplHard; // Number of MC events per cell in build-up
    const int  nBinHard; // Number of bins in build-up
    const int  EvPerBinHard; // Maximum events (equiv.) per bin in buid-up
    const double MaxWtRejHard; // Maximum wt for rejection, for OptRej=1
    //--------------------------------------------------------------------------------------------------
    //  General FOAM parameters
    const int  OptRej; // =0, weighted events;  =1, wt=1 events
    const int  OptDrive; // (D=2) Option, type of Drive =0,1,2 for TrueVol,Sigma,WtMax
    const int  Chat; // Chat level
    //--------------------------------------------------------------------------------------------------
    //  Parameters for branching
    double SigmaBorn;
    double SigmaVirt;
    double SigmaHard;
    double SigmaTot;
    //--------------------------------------------------------------------------------------------------
    const bool IEXPLBORN;
    const bool IEXPLVIRT;
    const bool IEXPLHARD;
    //--------------------------------------------------------------------------------------------------
    const bool PrintLHE;
    const bool PrintROOT;
    const bool PrintHard;
    //--------------------------------------------------------------------------------------------------
    const int verMajor,verMinor,verPatch;

private:
    Settings & cfg;
};

}
#endif // INFO_H
