************************************************************************
* This is the FORTRAN module (nc_ha_12_12__12_12) to calculate differential
* hard photon Bremsstrahlung for the Bhabha (el^+ + el^- -> el^+ + el^- process).
*
* Copyright (C), SANC Project Team, 2002-2020.
*
* http://sanc.jinr.ru/
* E-mail: <sanc@jinr.ru>
************************************************************************
      subroutine nc_ha_12_12__12_12_la0 (s,spr,ph4,eta4,eta5,hard)
      implicit none!
      include 's2n_declare.h'

      real*8 eta4,eta5
      real*8 kappaz,sq,sqspr
      real*8 p1p2,p1p3,p1p4,p1p5,p2p3,p2p4,p2p5,p3p4,p3p5,p4p5
      real*8 ph4,cosph4,sinph4,costh4,costh5,sinth4,sinth5
      complex*16 chizs,chizsc,chizspr,chizsprc
      complex*16 propizs,propzs,propzsc
      complex*16 propizspr,propzspr,propzsprc
      real*8 hardlam0,hardlam2,hardlam12,hardlamep
      REAL*8 tmpX(157)

      real*8 betaselel,ibetaselel,betasprelel,ibetasprelel
      complex*16 Iden023m,Idenz23m,Iden014m,Idenz14m
      complex*16 den023m,denz23m,den014m,denz14m
      complex*16 den023mc,denz23mc,den014mc,denz14mc
      real*8 MDZspr,MDZs,MDZs2spr,MDZ2spr,MDZ2s
      real*8 Iz1z2,Iz3z4


      hard = 0d0
      cmz2 = mz2

      sq = dsqrt(s)
      sqspr = dsqrt(spr)

      costh5 = (dexp(2d0*eta5)-1d0)/(dexp(2d0*eta5)+1d0)
      sinth5 = 2d0*dexp(eta5)/(dexp(2d0*eta5)+1d0)
      costh4 = (dexp(2d0*eta4)-1d0)/(dexp(2d0*eta4)+1d0)
      sinth4 = 2d0*dexp(eta4)/(dexp(2d0*eta4)+1d0)


      cosph4 = dcos(ph4)
      sinph4 = dsin(ph4)

      if (irun.eq.0) then
         propizs = (cmz2-s)
         propizspr = (cmz2-spr)
      else
         propizs = (dcmplx(mz**2,-s*wz/mz)-s)
         propizspr = (dcmplx(mz**2,-spr*wz/mz)-spr)
      endif
      propzs = 1d0/propizs
      propzspr = 1d0/propizspr
      propzsc = dconjg(propzs)
      propzsprc = dconjg(propzspr)

      betaselel = sqrt(1d0-4d0*relm2/s)
      ibetaselel = 1d0/betaselel
      betasprelel = sqrt(1d0-4d0*relm2/spr)
      ibetasprelel = 1d0/betasprelel

      p1p2 = -0.5d0*s+mel**2

      p1p3 = -0.25d0*sq*sqspr*betaselel*betasprelel*sinph4*sinth4*sinth5
     & -0.125d0*s*(1d0+betaselel*costh5)*(1d0-betasprelel*costh4)
     & -0.125d0*spr*(1d0-betaselel*costh5)*(1d0+betasprelel*costh4)

      p1p4 = 0.25d0*sq*sqspr*betaselel*betasprelel*sinph4*sinth4*sinth5
     & -0.125d0*s*(1d0+betaselel*costh5)*(1d0+betasprelel*costh4)
     & -0.125d0*spr*(1d0-betaselel*costh5)*(1d0-betasprelel*costh4)

      p1p5 = -0.25d0*(s-spr)*(1d0-betaselel*costh5)

      p2p3 = 0.25d0*sq*sqspr*betaselel*betasprelel*sinph4*sinth4*sinth5
     & -0.125d0*s*(1d0-betaselel*costh5)*(1d0-betasprelel*costh4)
     & -0.125d0*spr*(1d0+betaselel*costh5)*(1d0+betasprelel*costh4)

      p2p4 = -0.25d0*sq*sqspr*betaselel*betasprelel*sinph4*sinth4*sinth5
     & -0.125d0*s*(1d0-betaselel*costh5)*(1d0+betasprelel*costh4)
     & -0.125d0*spr*(1d0+betaselel*costh5)*(1d0-betasprelel*costh4)

      p2p5 = -0.25d0*(s-spr)*(1d0+betaselel*costh5)

      p3p4 = -0.5d0*spr+mel**2

      p3p5 = -0.25d0*(s-spr)*(1d0-betasprelel*costh4)

      p4p5 = -0.25d0*(s-spr)*(1d0+betasprelel*costh4)


      Iz1z2=0.25d0*(s-spr)/p1p5/p2p5
      Iz3z4=0.25d0*(s-spr)/p3p5/p4p5

      Idenz23m = cmz2-2*mel**2-2*p2p3
      Iden023m =     -2*mel**2-2*p2p3
      Idenz14m = cmz2-2*mel**2-2*p1p4
      Iden014m =     -2*mel**2-2*p1p4
      denz23m  = 1d0/Idenz23m
      den023m  = 1d0/Iden023m
      denz14m  = 1d0/Idenz14m
      den014m  = 1d0/Iden014m
      denz23mc = dconjg(denz23m)
      den023mc = dconjg(den023m)
      denz14mc = dconjg(denz14m)
      den014mc = dconjg(den014m)

      MDZspr=propzspr*propzsprc
      MDZs  =propzs*propzsc
      MDZs2spr=propzs*propzsprc+propzspr*propzsc
      MDZ2spr =propzspr+propzsprc
      MDZ2s   =propzs+propzsc


      tmpX(1)=pi**(-1)
      tmpX(2)=ctw**(-1)
      tmpX(3)=s**(-1)
      tmpX(4)=(s-spr)**(-1)
      tmpX(5)=spr**(-1)
      tmpX(6)=( - spr + s)
      tmpX(7)=tmpX(1)*alpha*g
      tmpX(7)=qel**4*tmpX(7)**2
      tmpX(8)=vmael**2
      tmpX(9)=tmpX(7)*tmpX(8)
      tmpX(10)=vpael**2
      tmpX(11)=tmpX(7)*tmpX(10)
      tmpX(12)=tmpX(9) + tmpX(11)
      tmpX(13)=denz14m + denz14mc
      tmpX(14)=tmpX(2)**2
      tmpX(15)=tmpX(14)*tmpX(13)
      tmpX(16)=tmpX(12)*tmpX(15)
      tmpX(17)=alpha**3*tmpX(1)*qel**6
      tmpX(18)=tmpX(17)*den014m
      tmpX(19)=tmpX(18) + 1.D0/64.D0*tmpX(16)
      tmpX(20)=tmpX(17)*tmpX(5)
      tmpX(21)=tmpX(19) - tmpX(20)
      tmpX(12)=tmpX(12)*tmpX(14)
      tmpX(22)=tmpX(12)*MDZ2spr
      tmpX(23)=1.D0/64.D0*tmpX(22)
      tmpX(24)=tmpX(21) + tmpX(23)
      tmpX(25)=tmpX(24)*tmpX(3)
      tmpX(26)=alpha*qel**2
      tmpX(27)=tmpX(26)*tmpX(1)**3
      tmpX(28)=vmael*g
      tmpX(28)=tmpX(27)*tmpX(28)**4
      tmpX(29)=vpael*g
      tmpX(29)=tmpX(27)*tmpX(29)**4
      tmpX(30)=tmpX(28) + tmpX(29)
      tmpX(31)=tmpX(2)**4
      tmpX(32)=tmpX(31)*denz14mc
      tmpX(33)=tmpX(30)*tmpX(32)
      tmpX(34)=tmpX(31)*denz14m
      tmpX(35)=tmpX(30)*tmpX(34)
      tmpX(36)=tmpX(33) - tmpX(35)
      tmpX(37)=tmpX(36)*propzs
      tmpX(38)=1.D0/16.D0*tmpX(34)
      tmpX(39)=tmpX(38)*tmpX(30)
      tmpX(40)=tmpX(12)*den014m
      tmpX(39)=tmpX(39) + tmpX(40)
      tmpX(41)=tmpX(39)*MDZ2s
      tmpX(42)= - tmpX(25) + 1.D0/1024.D0*tmpX(37) + 1.D0/64.D0*
     & tmpX(41)
      tmpX(43)=tmpX(30)*tmpX(31)
      tmpX(44)=tmpX(43)*propzspr
      tmpX(45)=tmpX(44) - tmpX(33)
      tmpX(45)= - tmpX(40) + 1.D0/16.D0*tmpX(45)
      tmpX(46)=tmpX(45)*denz23m
      tmpX(47)=tmpX(44) + tmpX(35)
      tmpX(47)=tmpX(40) + 1.D0/16.D0*tmpX(47)
      tmpX(48)=tmpX(47)*denz23mc
      tmpX(46)=tmpX(46) - tmpX(48)
      tmpX(48)=tmpX(36)*propzspr
      tmpX(49)=tmpX(34)*denz14mc
      tmpX(50)=tmpX(49)*tmpX(30)
      tmpX(51)=tmpX(43)*MDZspr
      tmpX(52)=tmpX(51) + tmpX(48) + tmpX(50)
      tmpX(53)=1.D0/32.D0*tmpX(16)
      tmpX(54)=tmpX(53) + tmpX(18)
      tmpX(54)=tmpX(54)*den014m
      tmpX(55)=tmpX(54) + 1.D0/512.D0*tmpX(52)
      tmpX(56)=2*tmpX(18)
      tmpX(53)=tmpX(53) + tmpX(56)
      tmpX(57)=1.D0/32.D0*tmpX(22)
      tmpX(58)=tmpX(53) + tmpX(57)
      tmpX(59)=tmpX(58) - tmpX(20)
      tmpX(60)=denz23mc + denz23m
      tmpX(61)=tmpX(12)*tmpX(60)
      tmpX(62)=1.D0/64.D0*tmpX(61)
      tmpX(12)=tmpX(12)*MDZ2s
      tmpX(63)=1.D0/64.D0*tmpX(12)
      tmpX(64)=tmpX(63) + tmpX(62) - tmpX(59)
      tmpX(64)=tmpX(5)*tmpX(64)
      tmpX(65)=tmpX(24)*den023m
      tmpX(66)=tmpX(31)*denz23m
      tmpX(67)=tmpX(66)*tmpX(30)
      tmpX(68)= - 1.D0/32.D0*tmpX(67) + tmpX(39)
      tmpX(69)=1.D0/32.D0*MDZ2spr
      tmpX(68)=tmpX(68)*tmpX(69)
      tmpX(70)=tmpX(43)*MDZs2spr
      tmpX(71)=1.D0/1024.D0*tmpX(70)
      tmpX(64)= - tmpX(65) + tmpX(64) + tmpX(68) - tmpX(71) - tmpX(42)
     &  + tmpX(55) + 1.D0/64.D0*tmpX(46)
      tmpX(68)=tmpX(3)**2
      tmpX(72)=tmpX(68)*p1p5
      tmpX(64)=tmpX(64)*tmpX(72)
      tmpX(73)=tmpX(39) + 1.D0/16.D0*tmpX(67)
      tmpX(73)=tmpX(73)*MDZ2spr
      tmpX(73)=tmpX(73) - tmpX(46)
      tmpX(74)=1.D0/2.D0*propzspr
      tmpX(75)=tmpX(36)*tmpX(74)
      tmpX(76)=tmpX(75) + tmpX(51)
      tmpX(76)=tmpX(73) + 1.D0/8.D0*tmpX(76)
      tmpX(21)=tmpX(21) + tmpX(57) + tmpX(62)
      tmpX(21)=tmpX(21)*tmpX(5)
      tmpX(21)= - 1.D0/64.D0*tmpX(76) + tmpX(21) - tmpX(65)
      tmpX(76)= - tmpX(3)*tmpX(21)
      tmpX(64)=tmpX(76) + tmpX(64)
      tmpX(64)=p1p3*tmpX(64)
      tmpX(76)=tmpX(39)*MDZ2spr
      tmpX(77)=tmpX(76) + tmpX(41)
      tmpX(78)=tmpX(37) + tmpX(70)
      tmpX(79)=g**4
      tmpX(27)=tmpX(27)*tmpX(79)
      tmpX(80)=tmpX(8)*tmpX(27)
      tmpX(81)=tmpX(80)*tmpX(10)
      tmpX(82)=tmpX(38)*tmpX(81)
      tmpX(83)=tmpX(7)*vpael
      tmpX(84)=tmpX(14)*vmael*tmpX(83)
      tmpX(85)=tmpX(84)*den014m
      tmpX(82)=tmpX(82) + tmpX(85)
      tmpX(86)=tmpX(82)*denz23mc
      tmpX(87)=1.D0/16.D0*tmpX(81)
      tmpX(88)=tmpX(87)*tmpX(32)
      tmpX(88)=tmpX(88) + tmpX(85)
      tmpX(88)=tmpX(88)*denz23m
      tmpX(86)=tmpX(86) + tmpX(88)
      tmpX(89)=1.D0/4.D0*tmpX(48)
      tmpX(90)=1.D0/2.D0*tmpX(80)
      tmpX(10)=tmpX(10)*tmpX(27)
      tmpX(27)=tmpX(90) - tmpX(10)
      tmpX(27)=tmpX(27)*tmpX(8)
      tmpX(91)=1.D0/2.D0*tmpX(29)
      tmpX(27)=tmpX(27) + tmpX(91)
      tmpX(92)=tmpX(27)*tmpX(49)
      tmpX(92)=tmpX(92) + tmpX(89)
      tmpX(93)=1.D0/2.D0*vmael
      tmpX(7)=tmpX(93)*tmpX(7)
      tmpX(94)=tmpX(83) - tmpX(7)
      tmpX(94)=tmpX(94)*vmael
      tmpX(95)=1.D0/2.D0*tmpX(11)
      tmpX(94)=tmpX(94) - tmpX(95)
      tmpX(96)=den014m*tmpX(15)*tmpX(94)
      tmpX(77)= - tmpX(96) + 1.D0/16.D0*tmpX(92) + 1.D0/2.D0*tmpX(86)
     &  + 1.D0/64.D0*tmpX(78) + 1.D0/4.D0*tmpX(77)
      tmpX(13)=tmpX(84)*tmpX(13)
      tmpX(92)=tmpX(18) + 1.D0/32.D0*tmpX(13)
      tmpX(96)=tmpX(92)*den023m
      tmpX(77)=tmpX(96) + 1.D0/16.D0*tmpX(77)
      tmpX(97)=tmpX(63) + tmpX(19)
      tmpX(98)=tmpX(97)*tmpX(5)
      tmpX(99)=tmpX(25) + tmpX(98) - tmpX(77)
      tmpX(100)=1.D0/2.D0*tmpX(3)
      tmpX(99)=tmpX(99)*tmpX(100)
      tmpX(10)=tmpX(90) + tmpX(10)
      tmpX(8)=tmpX(10)*tmpX(8)
      tmpX(8)=tmpX(8) + tmpX(91)
      tmpX(10)=tmpX(31)*MDZspr
      tmpX(90)=tmpX(8)*tmpX(10)
      tmpX(89)=tmpX(90) + tmpX(89)
      tmpX(73)=tmpX(73) + 1.D0/4.D0*tmpX(89)
      tmpX(89)=tmpX(62) + tmpX(19)
      tmpX(7)=tmpX(83) + tmpX(7)
      tmpX(7)=tmpX(7)*vmael
      tmpX(7)=tmpX(7) + tmpX(95)
      tmpX(83)=tmpX(7)*tmpX(14)
      tmpX(90)=MDZ2spr*tmpX(83)
      tmpX(91)=2*tmpX(20)
      tmpX(95)=1.D0/16.D0*tmpX(90) + tmpX(89) - tmpX(91)
      tmpX(95)=tmpX(95)*tmpX(5)
      tmpX(73)= - 1.D0/64.D0*tmpX(73) + tmpX(95) - tmpX(65)
      tmpX(95)=tmpX(68)*p2p3
      tmpX(101)=tmpX(73)*tmpX(95)
      tmpX(102)=MDZs2spr*tmpX(87)*tmpX(31)
      tmpX(103)=tmpX(84)*MDZ2s
      tmpX(104)=tmpX(103)*tmpX(5)
      tmpX(105)=tmpX(102) - tmpX(104)
      tmpX(106)=tmpX(84)*MDZ2spr
      tmpX(107)=1.D0/32.D0*tmpX(106)
      tmpX(108)=tmpX(107) - tmpX(20)
      tmpX(108)=tmpX(108)*tmpX(3)
      tmpX(109)= - tmpX(108) + 1.D0/32.D0*tmpX(105)
      tmpX(110)=tmpX(68)*p1p4
      tmpX(111)=tmpX(109)*tmpX(110)
      tmpX(99)=tmpX(111) + tmpX(99) + tmpX(101)
      tmpX(99)=p1p5*tmpX(99)
      tmpX(35)= - 1.D0/2.D0*tmpX(35) + tmpX(44)
      tmpX(101)=1.D0/8.D0*tmpX(67)
      tmpX(112)=1.D0/16.D0*propzs
      tmpX(113)=tmpX(112)*tmpX(43)
      tmpX(35)=tmpX(101) - tmpX(113) + 1.D0/8.D0*tmpX(35) - tmpX(40)
      tmpX(35)=denz23mc*tmpX(35)
      tmpX(114)= - 1.D0/2.D0*tmpX(70) + tmpX(51) - tmpX(75)
      tmpX(115)=tmpX(43)*MDZ2s
      tmpX(116)=tmpX(40) - tmpX(113) + 1.D0/16.D0*tmpX(115)
      tmpX(117)= - 1.D0/2.D0*tmpX(33) - tmpX(44)
      tmpX(117)=1.D0/8.D0*tmpX(117) - tmpX(116)
      tmpX(117)=denz23m*tmpX(117)
      tmpX(118)=tmpX(101) - tmpX(39)
      tmpX(118)=MDZ2spr*tmpX(118)
      tmpX(35)=tmpX(35) + tmpX(118) + 1.D0/8.D0*tmpX(114) + tmpX(117)
      tmpX(114)=1.D0/32.D0*tmpX(61)
      tmpX(57)= - tmpX(97) + tmpX(114) + tmpX(57)
      tmpX(117)=tmpX(20) - tmpX(57)
      tmpX(117)=tmpX(5)*tmpX(117)
      tmpX(118)=tmpX(17)*den023m
      tmpX(119)=tmpX(91) - tmpX(118)
      tmpX(57)= - tmpX(119) + tmpX(57)
      tmpX(57)=den023m*tmpX(57)
      tmpX(120)=tmpX(61) + tmpX(22)
      tmpX(121)=1.D0/64.D0*tmpX(120)
      tmpX(122)=tmpX(118) - tmpX(20)
      tmpX(123)=tmpX(121) + tmpX(122)
      tmpX(123)=tmpX(123)*tmpX(3)
      tmpX(35)=tmpX(123) + tmpX(57) + 1.D0/64.D0*tmpX(35) + tmpX(117)
      tmpX(57)=tmpX(68)*p1p3
      tmpX(35)=tmpX(35)*tmpX(57)
      tmpX(117)=1.D0/2.D0*tmpX(22)
      tmpX(124)=tmpX(61) + tmpX(117) + 1.D0/2.D0*tmpX(12)
      tmpX(125)=tmpX(122) + 1.D0/32.D0*tmpX(124)
      tmpX(125)=tmpX(125)*den023m
      tmpX(126)=tmpX(66)*MDZ2spr
      tmpX(30)=tmpX(126)*tmpX(30)
      tmpX(127)=tmpX(30) + tmpX(70)
      tmpX(128)=tmpX(43)*propzs
      tmpX(129)=tmpX(128) + tmpX(44)
      tmpX(130)=tmpX(129) - tmpX(115)
      tmpX(130)=tmpX(130)*denz23m
      tmpX(130)= - tmpX(130) + tmpX(127)
      tmpX(129)=tmpX(67) + 1.D0/2.D0*tmpX(129)
      tmpX(129)=tmpX(129)*denz23mc
      tmpX(129)=tmpX(129) + 1.D0/2.D0*tmpX(130)
      tmpX(130)=tmpX(61) + tmpX(12)
      tmpX(131)=tmpX(130)*tmpX(5)
      tmpX(129)= - tmpX(131) + 1.D0/8.D0*tmpX(129)
      tmpX(125)=tmpX(125) - tmpX(123) + 1.D0/64.D0*tmpX(129)
      tmpX(132)= - tmpX(125)*tmpX(110)
      tmpX(35)=tmpX(132) + tmpX(35)
      tmpX(35)=p2p5*tmpX(35)
      tmpX(35)=tmpX(35) + tmpX(99) + tmpX(64)
      tmpX(35)=p2p4*tmpX(35)
      tmpX(64)=tmpX(74)*tmpX(43)
      tmpX(99)=tmpX(81)*tmpX(34)
      tmpX(132)=tmpX(64) + tmpX(99)
      tmpX(27)=tmpX(27)*tmpX(66)
      tmpX(133)=1.D0/32.D0*tmpX(128)
      tmpX(27)=1.D0/16.D0*tmpX(132) + 1.D0/8.D0*tmpX(27) + tmpX(133) + 
     & tmpX(85)
      tmpX(27)=tmpX(27)*denz23mc
      tmpX(132)=tmpX(81)*tmpX(32)
      tmpX(64)=tmpX(64) - tmpX(132)
      tmpX(64)=1.D0/16.D0*tmpX(64) - 1.D0/32.D0*tmpX(115) + tmpX(133)
     &  - tmpX(85)
      tmpX(64)=tmpX(64)*denz23m
      tmpX(27)=tmpX(27) - tmpX(64) + 1.D0/32.D0*tmpX(127)
      tmpX(64)=1.D0/2.D0*tmpX(131) - tmpX(27)
      tmpX(94)=1.D0/16.D0*tmpX(94)
      tmpX(127)=tmpX(14)*tmpX(60)
      tmpX(94)=tmpX(127)*tmpX(94)
      tmpX(63)= - tmpX(94) + tmpX(23) + tmpX(63) + tmpX(92)
      tmpX(94)=tmpX(20) - tmpX(63)
      tmpX(94)=den023m*tmpX(94)
      tmpX(64)=tmpX(123) + 1.D0/32.D0*tmpX(64) + tmpX(94)
      tmpX(64)=tmpX(64)*tmpX(100)
      tmpX(73)=tmpX(73)*tmpX(110)
      tmpX(94)=tmpX(109)*tmpX(95)
      tmpX(64)=tmpX(73) + tmpX(64) + tmpX(94)
      tmpX(64)=p1p3*tmpX(64)
      tmpX(73)=tmpX(66)*tmpX(81)
      tmpX(73)=1.D0/8.D0*tmpX(73)
      tmpX(109)=tmpX(73) - tmpX(82)
      tmpX(109)=tmpX(109)*denz23mc
      tmpX(109)=tmpX(109) - tmpX(88)
      tmpX(133)=tmpX(105) + tmpX(109)
      tmpX(60)=tmpX(84)*tmpX(60)
      tmpX(84)=tmpX(118) + 1.D0/16.D0*tmpX(60)
      tmpX(134)=tmpX(84) - tmpX(92)
      tmpX(134)=tmpX(134)*den023m
      tmpX(133)= - tmpX(108) + 1.D0/32.D0*tmpX(133) + tmpX(134)
      tmpX(133)=tmpX(133)*tmpX(100)
      tmpX(135)=tmpX(106) + 1.D0/2.D0*tmpX(103)
      tmpX(135)= - tmpX(20) + 1.D0/16.D0*tmpX(135)
      tmpX(135)=tmpX(135)*tmpX(5)
      tmpX(136)=tmpX(81)*tmpX(31)
      tmpX(137)=tmpX(136)*MDZs2spr
      tmpX(138)=tmpX(136)*MDZspr
      tmpX(139)=tmpX(138) + 1.D0/2.D0*tmpX(137)
      tmpX(135)=tmpX(135) + tmpX(108) - 1.D0/256.D0*tmpX(139)
      tmpX(139)= - tmpX(135)*tmpX(95)
      tmpX(133)=tmpX(133) + tmpX(139)
      tmpX(133)=p1p4*tmpX(133)
      tmpX(73)=tmpX(73) + tmpX(82)
      tmpX(73)=tmpX(73)*denz23mc
      tmpX(73)=tmpX(73) + tmpX(88)
      tmpX(139)=tmpX(84) + tmpX(92)
      tmpX(139)=tmpX(139)*den023m
      tmpX(73)=tmpX(139) + 1.D0/32.D0*tmpX(73)
      tmpX(139)=spr*tmpX(3)
      tmpX(140)=tmpX(73)*tmpX(139)
      tmpX(140)=1.D0/4.D0*tmpX(140)
      tmpX(64)=tmpX(64) - tmpX(140) + tmpX(133)
      tmpX(64)=p2p5*tmpX(64)
      tmpX(21)=tmpX(21)*tmpX(100)
      tmpX(133)=1.D0/32.D0*tmpX(76)
      tmpX(59)=tmpX(5)*tmpX(59)
      tmpX(55)=tmpX(59) - tmpX(133) - tmpX(55)
      tmpX(55)=tmpX(55)*tmpX(72)
      tmpX(55)=tmpX(21) + tmpX(55)
      tmpX(55)=p1p3*tmpX(55)
      tmpX(59)=1.D0/16.D0*tmpX(106)
      tmpX(141)=tmpX(59) - tmpX(20)
      tmpX(141)=tmpX(141)*tmpX(5)
      tmpX(141)=tmpX(141) - 1.D0/256.D0*tmpX(138)
      tmpX(142)=p2p3*tmpX(100)
      tmpX(143)=p2p5*tmpX(95)
      tmpX(142)=tmpX(143) + tmpX(142)
      tmpX(142)=tmpX(141)*tmpX(142)
      tmpX(55)=tmpX(55) + tmpX(142)
      tmpX(55)=p4p5*tmpX(55)
      tmpX(44)=tmpX(67) + tmpX(44)
      tmpX(44)=tmpX(44)*denz23mc
      tmpX(142)=tmpX(67)*propzspr
      tmpX(44)=tmpX(44) + tmpX(51) - tmpX(142) + tmpX(30)
      tmpX(51)=1.D0/32.D0*tmpX(120)
      tmpX(142)=tmpX(51) - tmpX(20)
      tmpX(142)=tmpX(5)*tmpX(142)
      tmpX(119)= - tmpX(51) + tmpX(119)
      tmpX(119)=den023m*tmpX(119)
      tmpX(119)=tmpX(119) - 1.D0/512.D0*tmpX(44) + tmpX(142)
      tmpX(142)=tmpX(68)*p2p5
      tmpX(119)=tmpX(119)*tmpX(142)
      tmpX(21)=tmpX(21) + tmpX(119)
      tmpX(21)=p2p4*tmpX(21)
      tmpX(119)=p1p4*tmpX(100)
      tmpX(143)=p1p5*tmpX(110)
      tmpX(119)=tmpX(119) + tmpX(143)
      tmpX(119)=tmpX(141)*tmpX(119)
      tmpX(21)=tmpX(21) + tmpX(119)
      tmpX(21)=p3p5*tmpX(21)
      tmpX(71)= - tmpX(98) + tmpX(71) + 1.D0/64.D0*tmpX(76)
      tmpX(119)=tmpX(75) + tmpX(50)
      tmpX(42)=tmpX(54) + 1.D0/512.D0*tmpX(119) + tmpX(42) + tmpX(71)
      tmpX(143)=p1p5*p2p3
      tmpX(144)=tmpX(143)*tmpX(57)
      tmpX(145)= - tmpX(42)*tmpX(144)
      tmpX(96)=tmpX(96) + 1.D0/32.D0*tmpX(86)
      tmpX(146)=1.D0/16.D0*tmpX(13)
      tmpX(147)=tmpX(146) + tmpX(18)
      tmpX(147)=tmpX(147)*den014m
      tmpX(148)=1.D0/256.D0*tmpX(81)
      tmpX(149)=tmpX(148)*tmpX(49)
      tmpX(147)=tmpX(147) + tmpX(149)
      tmpX(149)=tmpX(96) + tmpX(147)
      tmpX(150)=tmpX(100)*spr
      tmpX(149)=tmpX(149)*tmpX(150)
      tmpX(151)=tmpX(96) - tmpX(147)
      tmpX(137)= - tmpX(151) + 1.D0/512.D0*tmpX(137)
      tmpX(104)= - tmpX(108) - 1.D0/32.D0*tmpX(104) + tmpX(137)
      tmpX(108)=p2p3*tmpX(3)
      tmpX(104)=tmpX(104)*tmpX(108)
      tmpX(104)= - tmpX(149) + tmpX(104)
      tmpX(152)=tmpX(95)*p1p4
      tmpX(135)= - tmpX(135)*tmpX(152)
      tmpX(104)=1.D0/2.D0*tmpX(104) + tmpX(135)
      tmpX(104)=p1p5*tmpX(104)
      tmpX(135)=spr*tmpX(96)
      tmpX(141)= - p1p4*tmpX(141)*tmpX(108)
      tmpX(21)=tmpX(21) + tmpX(35) + tmpX(55) + tmpX(64) + tmpX(145) + 
     & tmpX(104) - 1.D0/4.D0*tmpX(135) + tmpX(141)
      tmpX(21)=Iz1z2*tmpX(21)
      tmpX(33)=tmpX(116) + 1.D0/16.D0*tmpX(33)
      tmpX(33)=tmpX(33)*denz23m
      tmpX(35)=tmpX(113) + tmpX(39)
      tmpX(35)=tmpX(35)*denz23mc
      tmpX(33)=tmpX(35) + tmpX(33)
      tmpX(35)=1.D0/512.D0*tmpX(37) + 1.D0/32.D0*tmpX(41)
      tmpX(39)=tmpX(43)*MDZs
      tmpX(43)=1.D0/512.D0*tmpX(39)
      tmpX(54)=tmpX(43) + tmpX(35) + tmpX(54)
      tmpX(55)=1.D0/32.D0*tmpX(12)
      tmpX(17)=tmpX(17)*tmpX(3)
      tmpX(64)=tmpX(55) - tmpX(17)
      tmpX(104)=tmpX(64) + tmpX(53)
      tmpX(62)= - tmpX(23) - tmpX(62) - tmpX(122) + tmpX(104)
      tmpX(62)=tmpX(3)*tmpX(62)
      tmpX(113)=tmpX(97)*den023m
      tmpX(75)= - tmpX(50) + tmpX(75)
      tmpX(62)=tmpX(62) + tmpX(113) + 1.D0/512.D0*tmpX(75) + tmpX(71)
     &  - tmpX(54) + 1.D0/64.D0*tmpX(33)
      tmpX(62)=tmpX(62)*tmpX(57)
      tmpX(42)=tmpX(42)*tmpX(95)
      tmpX(71)=tmpX(3)*tmpX(104)
      tmpX(54)=tmpX(71) - 1.D0/512.D0*tmpX(50) - tmpX(54)
      tmpX(54)=tmpX(54)*tmpX(72)
      tmpX(42)=tmpX(62) + tmpX(42) + tmpX(54)
      tmpX(42)=p4p5*tmpX(42)
      tmpX(36)=tmpX(36)*tmpX(112)
      tmpX(36)=tmpX(41) + tmpX(36) + tmpX(33)
      tmpX(36)=tmpX(113) + 1.D0/64.D0*tmpX(36)
      tmpX(54)=tmpX(89) + tmpX(118)
      tmpX(62)=tmpX(54) + tmpX(64)
      tmpX(62)=tmpX(62)*tmpX(3)
      tmpX(62)=tmpX(62) - tmpX(36) - tmpX(43)
      tmpX(71)=tmpX(68)*spr
      tmpX(75)=tmpX(71)*p1p5
      tmpX(89)=tmpX(57)*spr
      tmpX(75)=tmpX(89) + 1.D0/2.D0*tmpX(75)
      tmpX(104)= - tmpX(62)*tmpX(75)
      tmpX(42)=tmpX(42) + tmpX(104)
      tmpX(42)=p2p4*tmpX(42)
      tmpX(58)=tmpX(58) - tmpX(91)
      tmpX(58)=tmpX(58)*tmpX(3)
      tmpX(53)=tmpX(53) + tmpX(55)
      tmpX(53)=tmpX(53)*tmpX(5)
      tmpX(104)=1.D0/16.D0*tmpX(16)
      tmpX(116)=tmpX(104) + tmpX(56)
      tmpX(116)=tmpX(116)*den014m
      tmpX(35)=1.D0/256.D0*tmpX(119) - tmpX(58) + 1.D0/512.D0*tmpX(70)
     &  + tmpX(133) + tmpX(35) + tmpX(116) - tmpX(53)
      tmpX(53)=p2p4*p1p5
      tmpX(58)=tmpX(53)*tmpX(95)
      tmpX(58)=tmpX(58) - tmpX(144)
      tmpX(35)=tmpX(35)*tmpX(58)
      tmpX(58)=tmpX(96)*tmpX(71)
      tmpX(116)=tmpX(96)*tmpX(3)
      tmpX(58)=tmpX(58) - tmpX(116)
      tmpX(59)=tmpX(59) - tmpX(91)
      tmpX(122)=tmpX(59)*tmpX(3)
      tmpX(122)= - tmpX(122) + 1.D0/16.D0*tmpX(105)
      tmpX(133)=tmpX(110) - tmpX(95)
      tmpX(122)=tmpX(133)*tmpX(122)
      tmpX(133)= - tmpX(122) + tmpX(58)
      tmpX(135)=p2p5*p1p3
      tmpX(133)=tmpX(133)*tmpX(135)
      tmpX(141)=tmpX(147)*tmpX(3)
      tmpX(145)=tmpX(147)*tmpX(71)
      tmpX(153)=tmpX(141) - tmpX(145)
      tmpX(153)=tmpX(153)*tmpX(143)
      tmpX(35)=tmpX(133) + tmpX(153) + tmpX(35)
      tmpX(35)=p4p5*tmpX(35)
      tmpX(133)=2*tmpX(118)
      tmpX(91)=tmpX(133) - tmpX(91)
      tmpX(153)=tmpX(91) + 1.D0/16.D0*tmpX(124)
      tmpX(153)=tmpX(153)*den023m
      tmpX(51)=tmpX(51) + tmpX(91)
      tmpX(51)=tmpX(51)*tmpX(3)
      tmpX(51)= - tmpX(153) + tmpX(51) - 1.D0/32.D0*tmpX(129)
      tmpX(153)= - tmpX(51)*tmpX(57)
      tmpX(84)=tmpX(84)*den023m
      tmpX(154)=tmpX(66)*denz23mc
      tmpX(148)=tmpX(154)*tmpX(148)
      tmpX(84)=tmpX(84) + tmpX(148)
      tmpX(148)=tmpX(84)*tmpX(3)
      tmpX(155)=tmpX(84)*tmpX(71)
      tmpX(153)=tmpX(153) + tmpX(148) - tmpX(155)
      tmpX(156)=p2p5*p1p4
      tmpX(153)=tmpX(153)*tmpX(156)
      tmpX(157)=tmpX(110)*p2p5
      tmpX(51)=tmpX(51)*tmpX(157)
      tmpX(58)=tmpX(122) + tmpX(58)
      tmpX(58)=p1p5*tmpX(58)
      tmpX(51)=tmpX(58) + tmpX(51)
      tmpX(51)=p2p4*tmpX(51)
      tmpX(51)=tmpX(153) + tmpX(51)
      tmpX(51)=p3p5*tmpX(51)
      tmpX(35)=tmpX(51) + tmpX(35)
      tmpX(35)=Iz1z2*tmpX(35)
      tmpX(30)=tmpX(78) + tmpX(30)
      tmpX(30)=tmpX(41) - tmpX(131) + 1.D0/16.D0*tmpX(30)
      tmpX(51)=1.D0/8.D0*tmpX(128)
      tmpX(45)= - 1.D0/8.D0*tmpX(115) + tmpX(51) - tmpX(45)
      tmpX(45)=denz23m*tmpX(45)
      tmpX(47)= - tmpX(101) - tmpX(51) + tmpX(47)
      tmpX(47)=denz23mc*tmpX(47)
      tmpX(45)=tmpX(47) + tmpX(45) + tmpX(30)
      tmpX(24)=tmpX(114) - tmpX(24)
      tmpX(47)= - tmpX(118) - tmpX(55) - tmpX(24)
      tmpX(47)=den023m*tmpX(47)
      tmpX(24)=tmpX(133) + tmpX(64) + tmpX(24)
      tmpX(24)=tmpX(3)*tmpX(24)
      tmpX(24)=tmpX(24) - tmpX(43) + 1.D0/64.D0*tmpX(45) + tmpX(47)
      tmpX(24)=tmpX(24)*tmpX(57)
      tmpX(45)=den023m*tmpX(63)
      tmpX(47)= - tmpX(121) - tmpX(118)
      tmpX(47)=tmpX(3)*tmpX(47)
      tmpX(27)=tmpX(47) + 1.D0/32.D0*tmpX(27) + tmpX(45)
      tmpX(27)=tmpX(27)*tmpX(71)
      tmpX(45)=tmpX(17) - 1.D0/64.D0*tmpX(130) - tmpX(118)
      tmpX(45)=tmpX(45)*tmpX(68)
      tmpX(27)=tmpX(45) + tmpX(27)
      tmpX(45)=1.D0/16.D0*MDZ2s
      tmpX(45)=tmpX(45)*tmpX(83)
      tmpX(45)= - tmpX(45) - tmpX(54) + 2*tmpX(17)
      tmpX(45)=tmpX(45)*tmpX(3)
      tmpX(47)=1.D0/256.D0*MDZs
      tmpX(8)=tmpX(47)*tmpX(8)*tmpX(31)
      tmpX(8)=tmpX(45) + tmpX(8) + tmpX(36)
      tmpX(36)=tmpX(8)*tmpX(110)
      tmpX(24)=tmpX(24) + tmpX(36) + 1.D0/2.D0*tmpX(27) - tmpX(94)
      tmpX(24)=p2p4*tmpX(24)
      tmpX(27)=tmpX(125)*tmpX(57)
      tmpX(36)= - tmpX(102) - tmpX(109)
      tmpX(45)=tmpX(107)*tmpX(3)
      tmpX(36)=tmpX(45) + 1.D0/32.D0*tmpX(36) - tmpX(134)
      tmpX(36)=tmpX(36)*tmpX(71)
      tmpX(47)= - tmpX(17) + 1.D0/32.D0*tmpX(103)
      tmpX(47)=tmpX(47)*tmpX(68)
      tmpX(36)=tmpX(47) + tmpX(36)
      tmpX(51)=tmpX(103) + 1.D0/2.D0*tmpX(106)
      tmpX(51)= - tmpX(17) - tmpX(20) + 1.D0/16.D0*tmpX(51)
      tmpX(51)=tmpX(51)*tmpX(3)
      tmpX(54)=tmpX(136)*MDZs
      tmpX(55)=tmpX(105) + 1.D0/8.D0*tmpX(54)
      tmpX(51)=tmpX(51) - 1.D0/32.D0*tmpX(55)
      tmpX(55)=tmpX(51)*tmpX(95)
      tmpX(27)=tmpX(27) + 1.D0/2.D0*tmpX(36) + tmpX(55)
      tmpX(27)=p1p4*tmpX(27)
      tmpX(36)=tmpX(115) - tmpX(128)
      tmpX(36)=tmpX(36)*denz23m
      tmpX(55)=tmpX(128) + tmpX(67)
      tmpX(55)=tmpX(55)*denz23mc
      tmpX(36)=tmpX(36) + tmpX(55)
      tmpX(55)=1.D0/32.D0*tmpX(130)
      tmpX(58)= - tmpX(55) - tmpX(118)
      tmpX(58)=den023m*tmpX(58)
      tmpX(55)= - tmpX(17) + tmpX(55) + tmpX(133)
      tmpX(55)=tmpX(3)*tmpX(55)
      tmpX(43)=tmpX(55) - tmpX(43) - 1.D0/512.D0*tmpX(36) + tmpX(58)
      tmpX(55)=tmpX(57)*p2p5
      tmpX(43)=tmpX(43)*tmpX(55)
      tmpX(58)= - tmpX(17) + 1.D0/16.D0*tmpX(103)
      tmpX(58)=tmpX(58)*tmpX(3)
      tmpX(58)=tmpX(58) - 1.D0/256.D0*tmpX(54)
      tmpX(63)=tmpX(95)*p1p5
      tmpX(64)=tmpX(58)*tmpX(63)
      tmpX(24)=tmpX(24) + tmpX(43) + tmpX(64) + tmpX(140) + tmpX(27)
      tmpX(24)=p3p5*tmpX(24)
      tmpX(19)= - tmpX(23) - tmpX(19)
      tmpX(19)=tmpX(3)*tmpX(19)
      tmpX(19)=tmpX(19) + tmpX(77)
      tmpX(19)=tmpX(19)*tmpX(71)
      tmpX(23)=tmpX(17) - tmpX(97)
      tmpX(23)=tmpX(23)*tmpX(68)
      tmpX(19)=tmpX(23) + tmpX(19)
      tmpX(8)=tmpX(8)*tmpX(95)
      tmpX(8)= - tmpX(111) + 1.D0/2.D0*tmpX(19) + tmpX(8)
      tmpX(8)=p1p3*tmpX(8)
      tmpX(19)=tmpX(45) - tmpX(137)
      tmpX(19)=tmpX(19)*tmpX(71)
      tmpX(19)=tmpX(47) + tmpX(19)
      tmpX(19)=p2p3*tmpX(19)
      tmpX(19)=tmpX(149) + tmpX(19)
      tmpX(23)=tmpX(51)*tmpX(152)
      tmpX(27)=tmpX(58)*tmpX(157)
      tmpX(8)=tmpX(27) + tmpX(8) + 1.D0/2.D0*tmpX(19) + tmpX(23)
      tmpX(8)=p4p5*tmpX(8)
      tmpX(19)= - tmpX(62)*tmpX(89)
      tmpX(23)= - p1p4*tmpX(58)*tmpX(71)
      tmpX(19)=tmpX(23) + tmpX(19)
      tmpX(19)=p2p5*tmpX(19)
      tmpX(23)=spr**2
      tmpX(27)=tmpX(23)*tmpX(116)
      tmpX(43)=1.D0/2.D0*spr
      tmpX(45)= - tmpX(63)*tmpX(43)
      tmpX(47)= - spr*tmpX(152)
      tmpX(45)=tmpX(45) + tmpX(47)
      tmpX(45)=tmpX(58)*tmpX(45)
      tmpX(8)=tmpX(35) + tmpX(24) + tmpX(42) + tmpX(8) + 1.D0/2.D0*
     & tmpX(19) - 1.D0/4.D0*tmpX(27) + tmpX(45)
      tmpX(8)=Iz3z4*tmpX(8)
      tmpX(19)=1.D0/128.D0*tmpX(37) + 1.D0/8.D0*tmpX(41)
      tmpX(24)=1.D0/8.D0*tmpX(76)
      tmpX(27)=1.D0/8.D0*tmpX(16)
      tmpX(35)=8*tmpX(18)
      tmpX(37)=tmpX(35) + tmpX(27)
      tmpX(12)=tmpX(37) + 1.D0/8.D0*tmpX(12)
      tmpX(41)=tmpX(5)*tmpX(12)
      tmpX(22)=tmpX(37) + 1.D0/8.D0*tmpX(22)
      tmpX(42)=8*tmpX(20)
      tmpX(45)=tmpX(22) - tmpX(42)
      tmpX(47)=tmpX(3)*tmpX(45)
      tmpX(16)= - 1.D0/4.D0*tmpX(16) - tmpX(35)
      tmpX(16)=den014m*tmpX(16)
      tmpX(16)=tmpX(47) + tmpX(41) - tmpX(24) - 1.D0/128.D0*tmpX(70) - 
     & 1.D0/64.D0*tmpX(119) + tmpX(16) - tmpX(19)
      tmpX(16)=tmpX(16)*tmpX(144)
      tmpX(35)=tmpX(48) + tmpX(70)
      tmpX(33)=tmpX(33) + tmpX(76) + 1.D0/16.D0*tmpX(35)
      tmpX(35)=1.D0/8.D0*tmpX(120)
      tmpX(41)=tmpX(35) - tmpX(42)
      tmpX(47)=8*tmpX(118)
      tmpX(48)=tmpX(47) + tmpX(41)
      tmpX(48)=tmpX(48)*tmpX(3)
      tmpX(51)=den023m - tmpX(5)
      tmpX(58)=tmpX(12)*tmpX(51)
      tmpX(58)= - tmpX(48) + 1.D0/8.D0*tmpX(33) + tmpX(58)
      tmpX(62)=tmpX(142)*p1p3**2
      tmpX(58)=tmpX(58)*tmpX(62)
      tmpX(16)=tmpX(16) + tmpX(58)
      tmpX(16)=p2p4*tmpX(16)
      tmpX(58)=1.D0/8.D0*tmpX(13)
      tmpX(64)=tmpX(58) + tmpX(56)
      tmpX(64)=tmpX(64)*den014m
      tmpX(67)=1.D0/128.D0*tmpX(81)
      tmpX(70)=tmpX(67)*tmpX(49)
      tmpX(64)=tmpX(64) + tmpX(70)
      tmpX(70)=tmpX(64)*tmpX(139)
      tmpX(76)=p2p3*tmpX(70)
      tmpX(77)=1.D0/4.D0*tmpX(106)
      tmpX(78)=tmpX(77) - tmpX(42)
      tmpX(83)=tmpX(78)*tmpX(3)
      tmpX(83)= - tmpX(83) + 1.D0/4.D0*tmpX(105)
      tmpX(97)= - tmpX(83)*p2p3**2*tmpX(110)
      tmpX(76)=tmpX(76) + tmpX(97)
      tmpX(76)=p1p5*tmpX(76)
      tmpX(97)=tmpX(56) + tmpX(146)
      tmpX(101)=tmpX(97)*den023m
      tmpX(101)=tmpX(101) + 1.D0/16.D0*tmpX(86)
      tmpX(102)=tmpX(101)*tmpX(139)
      tmpX(83)=tmpX(83)*tmpX(152)
      tmpX(102)=tmpX(102) - tmpX(83)
      tmpX(105)= - tmpX(102)*tmpX(135)
      tmpX(16)=tmpX(16) + tmpX(76) + tmpX(105)
      tmpX(16)=p4p5*tmpX(16)
      tmpX(76)= - tmpX(47) - 1.D0/4.D0*tmpX(124) + tmpX(42)
      tmpX(76)=den023m*tmpX(76)
      tmpX(48)=tmpX(48) - 1.D0/8.D0*tmpX(129) + tmpX(76)
      tmpX(48)=p1p4*tmpX(48)*tmpX(55)
      tmpX(30)=tmpX(30) - tmpX(46)
      tmpX(46)=tmpX(3) - den023m
      tmpX(45)= - tmpX(45)*tmpX(46)
      tmpX(45)=1.D0/8.D0*tmpX(30) + tmpX(45)
      tmpX(76)=tmpX(53)*tmpX(57)
      tmpX(45)=tmpX(45)*tmpX(76)
      tmpX(102)= - p1p5*tmpX(102)
      tmpX(45)=tmpX(45) + tmpX(102) + tmpX(48)
      tmpX(45)=p2p4*tmpX(45)
      tmpX(48)=tmpX(133) + 1.D0/8.D0*tmpX(60)
      tmpX(48)=tmpX(48)*den023m
      tmpX(67)=tmpX(67)*tmpX(154)
      tmpX(48)=tmpX(48) + tmpX(67)
      tmpX(67)=tmpX(48)*tmpX(139)
      tmpX(83)=tmpX(67) - tmpX(83)
      tmpX(83)=tmpX(83)*tmpX(156)
      tmpX(45)=tmpX(83) + tmpX(45)
      tmpX(45)=p3p5*tmpX(45)
      tmpX(16)=tmpX(45) + tmpX(16)
      tmpX(45)=Iz1z2*tmpX(4)*Iz3z4
      tmpX(16)=tmpX(16)*tmpX(45)
      tmpX(37)=1.D0/4.D0*tmpX(90) + tmpX(37) - tmpX(42)
      tmpX(37)=tmpX(37)*tmpX(5)
      tmpX(83)=vpael*tmpX(1)
      tmpX(26)=tmpX(79)*tmpX(26)*tmpX(83)**3
      tmpX(79)=tmpX(80)*vpael
      tmpX(80)=tmpX(26) + tmpX(79)
      tmpX(83)=tmpX(80)*vmael
      tmpX(102)=tmpX(83)*tmpX(32)
      tmpX(105)=tmpX(83)*tmpX(34)
      tmpX(102)=tmpX(102) - tmpX(105)
      tmpX(102)=tmpX(102)*tmpX(74)
      tmpX(107)=tmpX(10)*tmpX(83)
      tmpX(114)=tmpX(102) + tmpX(107)
      tmpX(85)=tmpX(85) + 1.D0/32.D0*tmpX(105)
      tmpX(85)=tmpX(85)*MDZ2spr
      tmpX(85)=tmpX(85) + 1.D0/16.D0*tmpX(114)
      tmpX(37)=tmpX(37) - 1.D0/4.D0*tmpX(85)
      tmpX(105)=tmpX(3)**3
      tmpX(37)=tmpX(37)*tmpX(105)*p2p3
      tmpX(114)=tmpX(83)*tmpX(38)
      tmpX(40)=tmpX(114) + tmpX(40)
      tmpX(40)=tmpX(40)*tmpX(69)
      tmpX(69)=tmpX(83)*tmpX(49)
      tmpX(69)=tmpX(69) + tmpX(102)
      tmpX(102)=tmpX(7)*tmpX(15)
      tmpX(56)=tmpX(56) + 1.D0/16.D0*tmpX(102)
      tmpX(56)=tmpX(56)*den014m
      tmpX(97)=tmpX(97)*tmpX(5)
      tmpX(40)=tmpX(56) - tmpX(97) + tmpX(40) + 1.D0/256.D0*tmpX(69)
      tmpX(56)=tmpX(40)*tmpX(68)
      tmpX(37)=tmpX(37) + tmpX(56)
      tmpX(56)=p1p5**2
      tmpX(37)=tmpX(37)*tmpX(56)
      tmpX(20)=4*tmpX(20)
      tmpX(35)=tmpX(35) - tmpX(20)
      tmpX(35)=tmpX(35)*tmpX(5)
      tmpX(69)=4*tmpX(118)
      tmpX(41)=tmpX(41) + tmpX(69)
      tmpX(41)=tmpX(41)*den023m
      tmpX(35)= - tmpX(35) + tmpX(41) + 1.D0/128.D0*tmpX(44)
      tmpX(41)=p2p5**2
      tmpX(44)=tmpX(41)*tmpX(105)
      tmpX(102)=tmpX(35)*tmpX(44)
      tmpX(114)=tmpX(102)*p1p3
      tmpX(22)=tmpX(22) - tmpX(20)
      tmpX(22)=tmpX(22)*tmpX(5)
      tmpX(18)=4*tmpX(18)
      tmpX(27)=tmpX(18) + tmpX(27)
      tmpX(27)=tmpX(27)*den014m
      tmpX(22)=tmpX(22) - tmpX(27) - tmpX(24) - 1.D0/128.D0*tmpX(52)
      tmpX(24)=tmpX(105)*tmpX(56)
      tmpX(52)=tmpX(24)*p1p3
      tmpX(115)=tmpX(22)*tmpX(52)
      tmpX(37)=tmpX(37) + tmpX(114) - tmpX(115)
      tmpX(37)=tmpX(37)*p2p4
      tmpX(102)=tmpX(102)*p2p4
      tmpX(77)=tmpX(77) - tmpX(20)
      tmpX(77)=tmpX(77)*tmpX(5)
      tmpX(77)=tmpX(77) - 1.D0/64.D0*tmpX(138)
      tmpX(114)=p1p4*tmpX(77)*tmpX(24)
      tmpX(102)=tmpX(114) - tmpX(102)
      tmpX(102)=tmpX(102)*p3p5
      tmpX(114)=tmpX(77)*p2p3
      tmpX(116)=tmpX(114)*tmpX(44)
      tmpX(115)=tmpX(116) + tmpX(115)
      tmpX(115)=tmpX(115)*p4p5
      tmpX(24)=tmpX(24)*tmpX(147)
      tmpX(116)=tmpX(84)*tmpX(44)
      tmpX(24)=tmpX(24) + tmpX(116)
      tmpX(24)=tmpX(6)*tmpX(24)*spr
      tmpX(24)=tmpX(24) + tmpX(37) + tmpX(102) + tmpX(115)
      tmpX(37)=tmpX(90) + 1.D0/2.D0*tmpX(61)
      tmpX(42)= - tmpX(42) + 1.D0/4.D0*tmpX(37)
      tmpX(42)=tmpX(42)*tmpX(5)
      tmpX(61)=tmpX(80)*tmpX(93)
      tmpX(80)=tmpX(66)*propzspr
      tmpX(80)=tmpX(80) - tmpX(126)
      tmpX(102)=tmpX(31)*propzspr
      tmpX(115)=tmpX(102)*denz23mc
      tmpX(115)=tmpX(80) - tmpX(115)
      tmpX(116)= - tmpX(61)*tmpX(115)
      tmpX(107)=tmpX(116) + tmpX(107)
      tmpX(78)=tmpX(78)*den023m
      tmpX(42)=tmpX(42) - tmpX(78) - 1.D0/64.D0*tmpX(107)
      tmpX(42)=tmpX(42)*tmpX(105)*p1p4
      tmpX(7)=tmpX(7)*tmpX(127)
      tmpX(7)=tmpX(7) + tmpX(117)
      tmpX(7)=tmpX(91) + 1.D0/16.D0*tmpX(7)
      tmpX(7)=tmpX(7)*den023m
      tmpX(78)=tmpX(83)*tmpX(66)
      tmpX(61)=tmpX(61)*tmpX(102)
      tmpX(61)=tmpX(78) + tmpX(61)
      tmpX(61)=tmpX(61)*denz23mc
      tmpX(78)= - tmpX(83)*tmpX(80)
      tmpX(61)=tmpX(61) + 1.D0/2.D0*tmpX(78)
      tmpX(78)=tmpX(60)*tmpX(5)
      tmpX(61)= - tmpX(78) + 1.D0/16.D0*tmpX(61)
      tmpX(7)=tmpX(7) + 1.D0/16.D0*tmpX(61)
      tmpX(61)=tmpX(7)*tmpX(68)
      tmpX(42)=tmpX(42) + tmpX(61)
      tmpX(42)=tmpX(42)*p1p3
      tmpX(61)= - tmpX(81)*tmpX(115)
      tmpX(61)= - tmpX(78) + 1.D0/16.D0*tmpX(61)
      tmpX(59)=tmpX(59)*den023m
      tmpX(59)=tmpX(59) + 1.D0/16.D0*tmpX(61)
      tmpX(61)=tmpX(59)*tmpX(68)
      tmpX(78)=tmpX(114)*tmpX(105)
      tmpX(61)=tmpX(61) + tmpX(78)
      tmpX(61)=tmpX(61)*p1p4
      tmpX(42)=tmpX(42) - tmpX(61)
      tmpX(61)= - tmpX(155) + tmpX(42)
      tmpX(61)=tmpX(61)*tmpX(41)
      tmpX(83)=tmpX(99) - tmpX(132)
      tmpX(91)=1.D0/16.D0*propzspr
      tmpX(83)=tmpX(83)*tmpX(91)
      tmpX(91)=tmpX(82)*MDZ2spr
      tmpX(83)=tmpX(83) - tmpX(91)
      tmpX(83)=tmpX(97) + 1.D0/16.D0*tmpX(83)
      tmpX(91)=tmpX(83)*tmpX(95)
      tmpX(78)=tmpX(78)*p1p4
      tmpX(78)=tmpX(78) - tmpX(91)
      tmpX(91)= - tmpX(145) - tmpX(78)
      tmpX(91)=tmpX(91)*tmpX(56)
      tmpX(61)=tmpX(91) + tmpX(61) + tmpX(24)
      tmpX(91)=Iz1z2**2
      tmpX(61)=tmpX(91)*tmpX(61)
      tmpX(97)=tmpX(56)*tmpX(57)
      tmpX(22)=tmpX(97)*tmpX(22)
      tmpX(99)=tmpX(41)*tmpX(57)
      tmpX(105)= - tmpX(35)*tmpX(99)
      tmpX(105)=tmpX(22) + tmpX(105)
      tmpX(105)=p2p4*tmpX(105)
      tmpX(90)= - tmpX(20) + tmpX(18) + tmpX(104) + 1.D0/8.D0*tmpX(90)
      tmpX(90)=tmpX(90)*tmpX(5)
      tmpX(85)=tmpX(90) - 1.D0/8.D0*tmpX(85)
      tmpX(90)=tmpX(85)*tmpX(95)
      tmpX(40)=tmpX(40)*tmpX(3)
      tmpX(40)=tmpX(90) + tmpX(40)
      tmpX(40)=tmpX(40)*tmpX(56)
      tmpX(37)= - tmpX(20) + 1.D0/8.D0*tmpX(37)
      tmpX(37)=tmpX(37)*tmpX(5)
      tmpX(20)= - tmpX(20) + 1.D0/8.D0*tmpX(106)
      tmpX(20)=tmpX(20)*den023m
      tmpX(20)=1.D0/128.D0*tmpX(107) + tmpX(20) - tmpX(37)
      tmpX(37)=tmpX(20)*tmpX(57)
      tmpX(59)=tmpX(59)*tmpX(3)
      tmpX(37)=tmpX(37) + tmpX(59)
      tmpX(59)=tmpX(37)*tmpX(41)
      tmpX(59)= - tmpX(40) + tmpX(59)
      tmpX(59)=p4p5*tmpX(59)
      tmpX(20)=tmpX(20)*tmpX(110)
      tmpX(7)=tmpX(7)*tmpX(3)
      tmpX(7)=tmpX(20) - tmpX(7)
      tmpX(7)=tmpX(7)*tmpX(41)
      tmpX(20)=tmpX(56)*tmpX(68)
      tmpX(85)=tmpX(85)*tmpX(20)
      tmpX(90)= - p2p4*tmpX(85)
      tmpX(83)=tmpX(83)*tmpX(3)
      tmpX(104)= - tmpX(56)*tmpX(83)
      tmpX(90)=tmpX(90) + tmpX(104) + tmpX(7)
      tmpX(90)=p3p5*tmpX(90)
      tmpX(104)=tmpX(147)*tmpX(139)
      tmpX(106)=tmpX(77)*tmpX(152)
      tmpX(107)=tmpX(104) + tmpX(106)
      tmpX(107)=tmpX(107)*tmpX(56)
      tmpX(84)=tmpX(84)*tmpX(139)
      tmpX(106)=tmpX(84) + tmpX(106)
      tmpX(106)=tmpX(106)*tmpX(41)
      tmpX(59)=tmpX(90) + tmpX(105) + tmpX(59) + tmpX(107) + tmpX(106)
      tmpX(59)=tmpX(59)*tmpX(91)
      tmpX(17)=4*tmpX(17)
      tmpX(12)=tmpX(12) - tmpX(17)
      tmpX(12)=tmpX(12)*tmpX(3)
      tmpX(39)=1.D0/128.D0*tmpX(39)
      tmpX(12)=tmpX(27) - tmpX(12) + 1.D0/128.D0*tmpX(50) + tmpX(19) + 
     & tmpX(39)
      tmpX(19)=tmpX(72) + tmpX(57)
      tmpX(19)=tmpX(19)*p2p4
      tmpX(12)= - tmpX(12)*tmpX(19)
      tmpX(27)= - tmpX(17) + 1.D0/4.D0*tmpX(103)
      tmpX(27)=tmpX(27)*tmpX(3)
      tmpX(27)=tmpX(27) - 1.D0/64.D0*tmpX(54)
      tmpX(50)=tmpX(157) + tmpX(152)
      tmpX(50)=tmpX(27)*tmpX(50)
      tmpX(12)=tmpX(12) + tmpX(104) + tmpX(50)
      tmpX(50)=p4p5**2
      tmpX(12)=tmpX(50)*tmpX(12)
      tmpX(54)=1.D0/8.D0*tmpX(130)
      tmpX(17)= - tmpX(17) + tmpX(54) + tmpX(47)
      tmpX(17)=tmpX(17)*tmpX(3)
      tmpX(47)=tmpX(54) + tmpX(69)
      tmpX(47)=tmpX(47)*den023m
      tmpX(17)= - tmpX(17) + tmpX(47) + tmpX(39) + 1.D0/128.D0*tmpX(36)
      tmpX(36)=tmpX(57)*p2p4
      tmpX(36)=tmpX(36) + tmpX(55)
      tmpX(17)= - tmpX(17)*tmpX(36)
      tmpX(39)=tmpX(63) + tmpX(152)
      tmpX(27)=tmpX(27)*tmpX(39)
      tmpX(17)=tmpX(84) + tmpX(27) + tmpX(17)
      tmpX(27)=p3p5**2
      tmpX(17)=tmpX(17)*tmpX(27)
      tmpX(12)=tmpX(17) + tmpX(12)
      tmpX(17)=Iz3z4**2
      tmpX(12)=tmpX(12)*tmpX(17)
      tmpX(12)=tmpX(59) + tmpX(12)
      tmpX(12)=tmpX(4)*tmpX(12)
      tmpX(39)=tmpX(50)*tmpX(141)
      tmpX(47)=tmpX(27)*tmpX(148)
      tmpX(39)=tmpX(39) + tmpX(47)
      tmpX(39)=tmpX(39)*tmpX(17)
      tmpX(12)=tmpX(12) + tmpX(39) + tmpX(61)
      tmpX(39)=mel**2
      tmpX(12)=tmpX(12)*tmpX(39)
      tmpX(33)=tmpX(123) - tmpX(113) - 1.D0/64.D0*tmpX(33) + tmpX(98)
      tmpX(47)=tmpX(68)*p2p4
      tmpX(33)=tmpX(33)*tmpX(47)
      tmpX(25)=tmpX(25) - 1.D0/64.D0*tmpX(30) - tmpX(65)
      tmpX(25)=tmpX(25)*tmpX(57)
      tmpX(30)=tmpX(87)*tmpX(66)
      tmpX(30)=tmpX(30) - tmpX(82)
      tmpX(54)=1.D0/16.D0*denz23mc
      tmpX(30)=tmpX(30)*tmpX(54)
      tmpX(30)=tmpX(30) + tmpX(147) - 1.D0/16.D0*tmpX(88)
      tmpX(54)=1.D0/2.D0*tmpX(118) - tmpX(92) + 1.D0/32.D0*tmpX(60)
      tmpX(54)=tmpX(54)*den023m
      tmpX(30)=tmpX(54) + 1.D0/2.D0*tmpX(30)
      tmpX(54)=tmpX(71) + tmpX(3)
      tmpX(30)=tmpX(54)*tmpX(30)
      tmpX(54)=tmpX(148)*p3p5
      tmpX(59)=tmpX(141)*p4p5
      tmpX(54)=tmpX(54) + tmpX(59)
      tmpX(54)=tmpX(54)*Iz3z4
      tmpX(30)=tmpX(54) + tmpX(30)
      tmpX(54)=tmpX(155)*p2p5
      tmpX(59)=tmpX(145)*p1p5
      tmpX(54)=tmpX(54) + tmpX(59)
      tmpX(59)=1.D0/2.D0*Iz1z2
      tmpX(61)=tmpX(54)*tmpX(59)
      tmpX(25)=tmpX(61) + tmpX(33) + tmpX(25) + tmpX(111) + tmpX(94) + 
     & 1.D0/2.D0*tmpX(30)
      tmpX(33)=1.D0/2.D0*tmpX(6)
      tmpX(25)=tmpX(25)*tmpX(33)
      tmpX(8)=tmpX(12) + tmpX(16) + tmpX(25) + tmpX(21) + tmpX(8)
      tmpX(12)=tmpX(28) - tmpX(29)
      tmpX(16)=tmpX(12)*tmpX(31)
      tmpX(21)=tmpX(16)*propzspr
      tmpX(25)=tmpX(12)*tmpX(32)
      tmpX(28)=tmpX(21) - tmpX(25)
      tmpX(9)=tmpX(9) - tmpX(11)
      tmpX(11)=tmpX(9)*tmpX(14)
      tmpX(14)=tmpX(11)*den014m
      tmpX(28)= - tmpX(14) + 1.D0/16.D0*tmpX(28)
      tmpX(29)=tmpX(28)*denz23m
      tmpX(31)=tmpX(12)*tmpX(34)
      tmpX(61)=tmpX(21) + tmpX(31)
      tmpX(61)=tmpX(14) + 1.D0/16.D0*tmpX(61)
      tmpX(63)=tmpX(61)*denz23mc
      tmpX(29)=tmpX(29) - tmpX(63)
      tmpX(63)=tmpX(25) - tmpX(31)
      tmpX(65)=tmpX(63)*propzspr
      tmpX(82)=tmpX(49)*tmpX(12)
      tmpX(84)=tmpX(16)*MDZspr
      tmpX(87)=tmpX(84) + tmpX(65) + tmpX(82)
      tmpX(9)=tmpX(9)*tmpX(15)
      tmpX(15)=tmpX(9)*den014m
      tmpX(87)=tmpX(15) + 1.D0/16.D0*tmpX(87)
      tmpX(88)=tmpX(63)*propzs
      tmpX(90)=tmpX(38)*tmpX(12)
      tmpX(92)=tmpX(90) + tmpX(14)
      tmpX(94)=tmpX(92)*MDZ2s
      tmpX(98)=1.D0/32.D0*tmpX(88) + 1.D0/2.D0*tmpX(94)
      tmpX(103)=tmpX(11)*denz23mc
      tmpX(104)=1.D0/2.D0*tmpX(103)
      tmpX(105)=tmpX(11)*denz23m
      tmpX(106)=tmpX(104) + 1.D0/2.D0*tmpX(105)
      tmpX(107)=tmpX(11)*MDZ2spr
      tmpX(111)=tmpX(107) + tmpX(9)
      tmpX(11)=tmpX(11)*MDZ2s
      tmpX(113)=1.D0/2.D0*tmpX(11)
      tmpX(114)=tmpX(113) - tmpX(111) + tmpX(106)
      tmpX(114)=tmpX(5)*tmpX(114)
      tmpX(116)=tmpX(111)*den023m
      tmpX(117)=1.D0/2.D0*tmpX(116)
      tmpX(118)=tmpX(111)*tmpX(100)
      tmpX(119)=tmpX(66)*tmpX(12)
      tmpX(120)= - 1.D0/32.D0*tmpX(119) + tmpX(92)
      tmpX(120)=MDZ2spr*tmpX(120)
      tmpX(121)=tmpX(16)*MDZs2spr
      tmpX(122)=1.D0/32.D0*tmpX(121)
      tmpX(114)= - tmpX(117) + tmpX(114) + tmpX(120) - tmpX(98) + 
     & tmpX(87) + tmpX(118) - tmpX(122) + 1.D0/2.D0*tmpX(29)
      tmpX(114)=tmpX(114)*tmpX(72)
      tmpX(120)=tmpX(63)*tmpX(74)
      tmpX(123)=tmpX(120) + tmpX(84)
      tmpX(124)=tmpX(92) + 1.D0/16.D0*tmpX(119)
      tmpX(124)=tmpX(124)*MDZ2spr
      tmpX(123)=tmpX(124) - tmpX(29) + 1.D0/8.D0*tmpX(123)
      tmpX(124)=tmpX(105) + tmpX(9)
      tmpX(104)=tmpX(104) + tmpX(107) + 1.D0/2.D0*tmpX(124)
      tmpX(104)=tmpX(104)*tmpX(5)
      tmpX(104)=tmpX(117) - tmpX(104) + 1.D0/2.D0*tmpX(123)
      tmpX(117)=tmpX(3)*tmpX(104)
      tmpX(114)=tmpX(117) + tmpX(114)
      tmpX(114)=p1p3*tmpX(114)
      tmpX(117)=tmpX(120) + tmpX(82)
      tmpX(98)=tmpX(98) - tmpX(118) + tmpX(15) + 1.D0/16.D0*tmpX(117)
      tmpX(117)=tmpX(11) + tmpX(9)
      tmpX(118)=1.D0/2.D0*tmpX(117)
      tmpX(123)=tmpX(118)*tmpX(5)
      tmpX(124)=tmpX(92)*MDZ2spr
      tmpX(122)=tmpX(122) + 1.D0/2.D0*tmpX(124)
      tmpX(123)=tmpX(123) - tmpX(122)
      tmpX(125)= - tmpX(123) + tmpX(98)
      tmpX(127)= - tmpX(125)*tmpX(100)
      tmpX(128)= - tmpX(104)*tmpX(95)
      tmpX(127)=tmpX(127) + tmpX(128)
      tmpX(127)=p1p5*tmpX(127)
      tmpX(31)= - 1.D0/2.D0*tmpX(31) + tmpX(21)
      tmpX(128)=1.D0/8.D0*tmpX(119)
      tmpX(129)=tmpX(112)*tmpX(16)
      tmpX(31)=tmpX(128) - tmpX(129) + 1.D0/8.D0*tmpX(31) - tmpX(14)
      tmpX(31)=denz23mc*tmpX(31)
      tmpX(130)= - 1.D0/2.D0*tmpX(121) + tmpX(84) - tmpX(120)
      tmpX(131)=tmpX(16)*MDZ2s
      tmpX(132)=tmpX(14) - tmpX(129) + 1.D0/16.D0*tmpX(131)
      tmpX(133)= - 1.D0/2.D0*tmpX(25) - tmpX(21)
      tmpX(133)=1.D0/8.D0*tmpX(133) - tmpX(132)
      tmpX(133)=denz23m*tmpX(133)
      tmpX(136)=tmpX(128) - tmpX(92)
      tmpX(136)=MDZ2spr*tmpX(136)
      tmpX(31)=tmpX(31) + tmpX(136) + 1.D0/8.D0*tmpX(130) + tmpX(133)
      tmpX(103)=tmpX(105) + tmpX(103)
      tmpX(105)=tmpX(103) + tmpX(107)
      tmpX(130)=tmpX(118) - tmpX(105)
      tmpX(51)= - tmpX(130)*tmpX(51)
      tmpX(130)=tmpX(105)*tmpX(100)
      tmpX(31)=tmpX(130) + 1.D0/2.D0*tmpX(31) + tmpX(51)
      tmpX(31)=tmpX(31)*tmpX(57)
      tmpX(51)=tmpX(16)*propzs
      tmpX(133)=tmpX(51) + tmpX(21)
      tmpX(136)=tmpX(133) - tmpX(131)
      tmpX(136)=tmpX(136)*denz23m
      tmpX(12)=tmpX(126)*tmpX(12)
      tmpX(126)=tmpX(136) - tmpX(12) - tmpX(121)
      tmpX(133)=tmpX(119) + 1.D0/2.D0*tmpX(133)
      tmpX(133)=tmpX(133)*denz23mc
      tmpX(126)= - tmpX(133) + 1.D0/2.D0*tmpX(126)
      tmpX(133)=tmpX(103) + tmpX(11)
      tmpX(136)=tmpX(133)*tmpX(5)
      tmpX(137)=tmpX(136) + 1.D0/8.D0*tmpX(126)
      tmpX(107)=1.D0/2.D0*tmpX(107)
      tmpX(103)=tmpX(103) + tmpX(113) + tmpX(107)
      tmpX(103)=tmpX(103)*den023m
      tmpX(103)=tmpX(103) - tmpX(130)
      tmpX(113)= - tmpX(103) + 1.D0/2.D0*tmpX(137)
      tmpX(130)=tmpX(113)*tmpX(110)
      tmpX(31)=tmpX(130) + tmpX(31)
      tmpX(31)=p2p5*tmpX(31)
      tmpX(31)=tmpX(31) + tmpX(127) + tmpX(114)
      tmpX(31)=p2p4*tmpX(31)
      tmpX(114)=tmpX(113)*tmpX(100)
      tmpX(127)= - tmpX(104)*tmpX(110)
      tmpX(114)=tmpX(114) + tmpX(127)
      tmpX(114)=tmpX(114)*tmpX(135)
      tmpX(104)=tmpX(104)*tmpX(100)
      tmpX(127)=tmpX(111)*tmpX(5)
      tmpX(87)= - tmpX(87) + tmpX(127) - tmpX(124)
      tmpX(130)=tmpX(87)*tmpX(72)
      tmpX(130)= - tmpX(104) + tmpX(130)
      tmpX(137)=p4p5*p1p3
      tmpX(130)=tmpX(130)*tmpX(137)
      tmpX(21)=tmpX(119) + tmpX(21)
      tmpX(21)=tmpX(21)*denz23mc
      tmpX(138)=tmpX(119)*propzspr
      tmpX(21)=tmpX(21) + tmpX(84) - tmpX(138) + tmpX(12)
      tmpX(84)=tmpX(105)*tmpX(5)
      tmpX(138)=tmpX(105)*den023m
      tmpX(21)= - tmpX(84) + tmpX(138) + 1.D0/16.D0*tmpX(21)
      tmpX(140)= - tmpX(21)*tmpX(142)
      tmpX(104)= - tmpX(104) + tmpX(140)
      tmpX(140)=p3p5*p2p4
      tmpX(104)=tmpX(104)*tmpX(140)
      tmpX(141)=tmpX(125)*tmpX(144)
      tmpX(31)=tmpX(104) + tmpX(31) + tmpX(130) - tmpX(141) + tmpX(114)
      tmpX(31)=tmpX(31)*tmpX(59)
      tmpX(25)=tmpX(132) + 1.D0/16.D0*tmpX(25)
      tmpX(25)=tmpX(25)*denz23m
      tmpX(59)=tmpX(129) + tmpX(92)
      tmpX(59)=tmpX(59)*denz23mc
      tmpX(25)=tmpX(59) + tmpX(25)
      tmpX(59)=tmpX(63)*tmpX(112)
      tmpX(59)=tmpX(59) + tmpX(94)
      tmpX(16)=tmpX(16)*MDZs
      tmpX(63)=1.D0/16.D0*tmpX(16)
      tmpX(92)=tmpX(15) + tmpX(63) + tmpX(59)
      tmpX(104)= - tmpX(107) + tmpX(117) - tmpX(106)
      tmpX(104)=tmpX(3)*tmpX(104)
      tmpX(82)= - tmpX(82) + tmpX(120)
      tmpX(112)=den023m*tmpX(118)
      tmpX(82)=tmpX(104) + tmpX(112) + 1.D0/16.D0*tmpX(82) - tmpX(123)
     &  - tmpX(92) + 1.D0/2.D0*tmpX(25)
      tmpX(82)=tmpX(82)*tmpX(57)
      tmpX(104)=tmpX(125)*tmpX(95)
      tmpX(112)=tmpX(117)*tmpX(3)
      tmpX(90)=tmpX(90)*denz14mc
      tmpX(90)=tmpX(92) + tmpX(90) - tmpX(112)
      tmpX(72)= - tmpX(90)*tmpX(72)
      tmpX(72)=tmpX(82) + tmpX(104) + tmpX(72)
      tmpX(72)=p4p5*tmpX(72)
      tmpX(82)=tmpX(117)*den023m
      tmpX(25)=tmpX(82) + tmpX(25)
      tmpX(16)=tmpX(59) + tmpX(25) + 1.D0/8.D0*tmpX(16)
      tmpX(9)=1.D0/2.D0*tmpX(9)
      tmpX(11)=tmpX(11) + tmpX(106) + tmpX(9)
      tmpX(11)=tmpX(11)*tmpX(3)
      tmpX(11)=tmpX(11) - 1.D0/2.D0*tmpX(16)
      tmpX(16)= - tmpX(11)*tmpX(75)
      tmpX(16)=tmpX(72) + tmpX(16)
      tmpX(16)=p2p4*tmpX(16)
      tmpX(12)=tmpX(12) + tmpX(88) + tmpX(121)
      tmpX(12)=tmpX(94) - tmpX(136) + 1.D0/16.D0*tmpX(12)
      tmpX(59)=1.D0/8.D0*tmpX(51)
      tmpX(28)= - 1.D0/8.D0*tmpX(131) + tmpX(59) - tmpX(28)
      tmpX(28)=denz23m*tmpX(28)
      tmpX(59)= - tmpX(128) - tmpX(59) + tmpX(61)
      tmpX(59)=denz23mc*tmpX(59)
      tmpX(28)=tmpX(59) + tmpX(28) + tmpX(12)
      tmpX(9)= - tmpX(133) + tmpX(9) + tmpX(107)
      tmpX(9)= - tmpX(9)*tmpX(46)
      tmpX(9)= - tmpX(63) + 1.D0/2.D0*tmpX(28) + tmpX(9)
      tmpX(9)=tmpX(9)*tmpX(57)
      tmpX(28)= - 1.D0/16.D0*tmpX(126) + tmpX(103)
      tmpX(28)=tmpX(28)*tmpX(71)
      tmpX(59)=tmpX(133)*tmpX(68)
      tmpX(28)= - 1.D0/2.D0*tmpX(59) + tmpX(28)
      tmpX(59)= - tmpX(11)*tmpX(110)
      tmpX(9)=tmpX(9) + 1.D0/2.D0*tmpX(28) + tmpX(59)
      tmpX(9)=p2p4*tmpX(9)
      tmpX(28)=tmpX(113)*p1p4
      tmpX(59)= - tmpX(57)*tmpX(28)
      tmpX(61)=tmpX(51) + tmpX(119)
      tmpX(61)=tmpX(61)*denz23mc
      tmpX(51)=tmpX(131) - tmpX(51)
      tmpX(51)=tmpX(51)*denz23m
      tmpX(51)=tmpX(61) + tmpX(51)
      tmpX(46)= - tmpX(133)*tmpX(46)
      tmpX(46)=tmpX(46) + tmpX(63) + 1.D0/16.D0*tmpX(51)
      tmpX(51)= - tmpX(46)*tmpX(55)
      tmpX(9)=tmpX(9) + tmpX(59) + tmpX(51)
      tmpX(9)=p3p5*tmpX(9)
      tmpX(51)=tmpX(122) + tmpX(98)
      tmpX(51)=tmpX(51)*tmpX(71)
      tmpX(59)= - tmpX(68)*tmpX(118)
      tmpX(51)=tmpX(59) + tmpX(51)
      tmpX(59)= - tmpX(11)*tmpX(95)
      tmpX(51)=1.D0/2.D0*tmpX(51) + tmpX(59)
      tmpX(51)=tmpX(51)*tmpX(137)
      tmpX(11)= - tmpX(11)*tmpX(55)*tmpX(43)
      tmpX(9)=tmpX(9) + tmpX(16) + tmpX(11) + tmpX(51)
      tmpX(11)=tmpX(53)*tmpX(104)
      tmpX(11)= - tmpX(141) + tmpX(11)
      tmpX(11)=p4p5*tmpX(11)
      tmpX(16)=tmpX(28)*tmpX(55)
      tmpX(28)=p2p4*tmpX(113)*tmpX(157)
      tmpX(28)= - tmpX(16) + tmpX(28)
      tmpX(28)=p3p5*tmpX(28)
      tmpX(11)=tmpX(28) + tmpX(11)
      tmpX(11)=Iz1z2*tmpX(11)
      tmpX(9)=1.D0/2.D0*tmpX(9) + tmpX(11)
      tmpX(9)=Iz3z4*tmpX(9)
      tmpX(11)=tmpX(65) + tmpX(121)
      tmpX(28)=tmpX(105)*tmpX(3)
      tmpX(51)=tmpX(117)*tmpX(5)
      tmpX(11)= - tmpX(51) + 1.D0/16.D0*tmpX(11) - tmpX(28) + tmpX(25)
     &  + tmpX(124)
      tmpX(25)= - tmpX(11)*tmpX(47)
      tmpX(28)=tmpX(111)*tmpX(3)
      tmpX(12)=tmpX(116) - tmpX(28) + tmpX(12) - tmpX(29)
      tmpX(28)= - tmpX(12)*tmpX(57)
      tmpX(25)=tmpX(28) + tmpX(25)
      tmpX(25)=tmpX(6)*tmpX(25)
      tmpX(9)=1.D0/8.D0*tmpX(25) + tmpX(31) + tmpX(9)
      tmpX(11)=tmpX(11)*tmpX(62)
      tmpX(11)= - tmpX(141) + 1.D0/2.D0*tmpX(11)
      tmpX(11)=p2p4*p4p5*tmpX(11)
      tmpX(12)=tmpX(12)*tmpX(76)
      tmpX(12)=tmpX(16) + 1.D0/2.D0*tmpX(12)
      tmpX(12)=tmpX(12)*tmpX(140)
      tmpX(11)=tmpX(11) + tmpX(12)
      tmpX(11)=tmpX(11)*tmpX(45)
      tmpX(12)=tmpX(97)*tmpX(87)
      tmpX(16)= - tmpX(21)*tmpX(99)
      tmpX(16)=tmpX(12) + tmpX(16)
      tmpX(16)=p2p4*tmpX(16)
      tmpX(25)=tmpX(26) - tmpX(79)
      tmpX(26)=tmpX(25)*vmael
      tmpX(28)=tmpX(26)*tmpX(38)
      tmpX(14)=tmpX(28) - tmpX(14)
      tmpX(14)=tmpX(14)*MDZ2spr
      tmpX(28)=tmpX(26)*tmpX(49)
      tmpX(29)=tmpX(34) - tmpX(32)
      tmpX(29)=tmpX(74)*tmpX(26)*tmpX(29)
      tmpX(28)=tmpX(28) - tmpX(29)
      tmpX(14)= - tmpX(15) + tmpX(14) + 1.D0/8.D0*tmpX(28)
      tmpX(15)=tmpX(14)*tmpX(100)
      tmpX(25)=tmpX(25)*tmpX(93)
      tmpX(28)=MDZ2spr*tmpX(25)*tmpX(34)
      tmpX(10)=tmpX(10)*tmpX(26)
      tmpX(28)= - tmpX(29) + tmpX(28) + tmpX(10)
      tmpX(28)=tmpX(127) + 1.D0/8.D0*tmpX(28)
      tmpX(29)=tmpX(28)*tmpX(95)
      tmpX(15)=tmpX(15) + tmpX(29)
      tmpX(29)=1.D0/2.D0*tmpX(56)
      tmpX(15)=tmpX(15)*tmpX(29)
      tmpX(29)=p4p5*tmpX(15)
      tmpX(31)=tmpX(41)*tmpX(68)
      tmpX(32)=tmpX(31)*tmpX(21)
      tmpX(28)=tmpX(28)*tmpX(20)
      tmpX(28)= - tmpX(32) + 1.D0/2.D0*tmpX(28)
      tmpX(32)= - tmpX(28)*tmpX(140)
      tmpX(16)=tmpX(32) + tmpX(29) + tmpX(16)
      tmpX(16)=tmpX(16)*tmpX(91)
      tmpX(19)= - tmpX(50)*tmpX(90)*tmpX(19)
      tmpX(29)= - tmpX(27)*tmpX(46)*tmpX(36)
      tmpX(19)=tmpX(19) + tmpX(29)
      tmpX(19)=tmpX(19)*tmpX(17)
      tmpX(16)=tmpX(16) + tmpX(19)
      tmpX(16)=tmpX(4)*tmpX(16)
      tmpX(19)=tmpX(87)*tmpX(52)
      tmpX(29)=tmpX(19)*p4p5
      tmpX(14)=tmpX(14)*tmpX(20)
      tmpX(14)=tmpX(19) + 1.D0/4.D0*tmpX(14)
      tmpX(19)= - p2p4*tmpX(14)
      tmpX(19)=tmpX(29) + tmpX(19)
      tmpX(19)=tmpX(19)*tmpX(91)
      tmpX(16)=tmpX(19) + tmpX(16)
      tmpX(16)=tmpX(16)*tmpX(39)
      tmpX(9)=1.D0/2.D0*tmpX(16) + 1.D0/4.D0*tmpX(9) + tmpX(11)
      tmpX(9)=1.D0/4.D0*tmpX(9)
      tmpX(11)=tmpX(115)*tmpX(25)
      tmpX(10)=tmpX(11) - tmpX(10)
      tmpX(10)= - tmpX(84) + 1.D0/8.D0*tmpX(10)
      tmpX(11)=tmpX(10)*tmpX(99)
      tmpX(11)= - 1.D0/2.D0*tmpX(11) + tmpX(15) + tmpX(12)
      tmpX(11)=p4p5*tmpX(11)
      tmpX(12)=tmpX(26)*tmpX(66)
      tmpX(15)=tmpX(25)*tmpX(102)
      tmpX(12)=tmpX(12) + tmpX(15)
      tmpX(12)=tmpX(12)*denz23mc
      tmpX(15)=tmpX(80)*tmpX(26)
      tmpX(12)=tmpX(12) - 1.D0/2.D0*tmpX(15)
      tmpX(12)= - tmpX(138) + 1.D0/8.D0*tmpX(12)
      tmpX(15)= - tmpX(12)*tmpX(100)
      tmpX(10)=tmpX(10)*tmpX(110)
      tmpX(10)=tmpX(15) + tmpX(10)
      tmpX(10)=tmpX(10)*tmpX(41)
      tmpX(15)= - p2p4*tmpX(28)
      tmpX(10)=1.D0/2.D0*tmpX(10) + tmpX(15)
      tmpX(10)=p3p5*tmpX(10)
      tmpX(10)=tmpX(11) + tmpX(10)
      tmpX(10)=tmpX(4)*tmpX(10)
      tmpX(11)=tmpX(21)*tmpX(44)
      tmpX(15)= - p1p3*tmpX(11)
      tmpX(14)=tmpX(15) - tmpX(14)
      tmpX(14)=p2p4*tmpX(14)
      tmpX(11)=tmpX(140)*tmpX(11)
      tmpX(12)=tmpX(12)*tmpX(99)
      tmpX(10)=tmpX(10) + tmpX(11) + tmpX(14) + 1.D0/4.D0*tmpX(12) + 
     & tmpX(29)
      tmpX(10)=1.D0/8.D0*tmpX(39)*tmpX(91)*tmpX(10)
      tmpX(11)=tmpX(48)*tmpX(71)
      tmpX(11)=tmpX(11) - tmpX(42)
      tmpX(11)=tmpX(11)*tmpX(41)
      tmpX(12)=tmpX(64)*tmpX(71)
      tmpX(12)=tmpX(12) + tmpX(78)
      tmpX(12)=tmpX(12)*tmpX(56)
      tmpX(11)=tmpX(12) + tmpX(11) - tmpX(24)
      tmpX(11)=tmpX(91)*tmpX(11)
      tmpX(12)= - tmpX(77)*tmpX(95)
      tmpX(12)=tmpX(12) - tmpX(37)
      tmpX(12)=tmpX(12)*tmpX(41)
      tmpX(12)=tmpX(12) + tmpX(40) - tmpX(22)
      tmpX(12)=p4p5*tmpX(12)
      tmpX(14)=tmpX(35)*tmpX(31)
      tmpX(14)=tmpX(85) + tmpX(14)
      tmpX(14)=p2p4*tmpX(14)
      tmpX(15)= - tmpX(77)*tmpX(110)
      tmpX(15)=tmpX(83) + tmpX(15)
      tmpX(15)=tmpX(15)*tmpX(56)
      tmpX(7)=tmpX(14) + tmpX(15) - tmpX(7)
      tmpX(7)=p3p5*tmpX(7)
      tmpX(14)= - tmpX(56)*tmpX(70)
      tmpX(15)= - tmpX(41)*tmpX(67)
      tmpX(7)=tmpX(7) + tmpX(12) + tmpX(14) + tmpX(15)
      tmpX(7)=tmpX(7)*tmpX(91)
      tmpX(12)= - tmpX(50)*tmpX(70)
      tmpX(14)= - tmpX(27)*tmpX(67)
      tmpX(12)=tmpX(12) + tmpX(14)
      tmpX(12)=tmpX(12)*tmpX(17)
      tmpX(7)=tmpX(7) + tmpX(12)
      tmpX(7)=tmpX(4)*tmpX(7)
      tmpX(12)= - tmpX(64)*tmpX(50)
      tmpX(14)= - tmpX(48)*tmpX(27)
      tmpX(12)=tmpX(12) + tmpX(14)
      tmpX(12)=tmpX(17)*tmpX(3)*tmpX(12)
      tmpX(7)=tmpX(7) + tmpX(12) + tmpX(11)
      tmpX(7)=tmpX(7)*tmpX(39)
      tmpX(11)=tmpX(151)*tmpX(108)
      tmpX(11)=tmpX(149) + tmpX(11)
      tmpX(11)=p1p5*tmpX(11)
      tmpX(12)=tmpX(73)*tmpX(150)
      tmpX(14)=tmpX(134) + 1.D0/32.D0*tmpX(109)
      tmpX(15)= - p1p3 - p1p4
      tmpX(15)=tmpX(14)*tmpX(3)*tmpX(15)
      tmpX(15)=tmpX(12) + tmpX(15)
      tmpX(15)=p2p5*tmpX(15)
      tmpX(16)=tmpX(96)*tmpX(43)
      tmpX(17)=tmpX(3)*tmpX(151)*tmpX(53)
      tmpX(11)=tmpX(17) + tmpX(15) + tmpX(16) + tmpX(11)
      tmpX(11)=Iz1z2*tmpX(11)
      tmpX(15)= - spr*tmpX(95)
      tmpX(15)= - tmpX(89) + tmpX(15)
      tmpX(15)=tmpX(151)*tmpX(15)
      tmpX(15)= - tmpX(149) + tmpX(15)
      tmpX(15)=p4p5*tmpX(15)
      tmpX(16)=p2p4 + p1p4
      tmpX(14)=tmpX(14)*tmpX(71)*tmpX(16)
      tmpX(12)= - tmpX(12) + tmpX(14)
      tmpX(12)=p3p5*tmpX(12)
      tmpX(14)=tmpX(71) - tmpX(3)
      tmpX(16)=tmpX(14)*tmpX(101)
      tmpX(17)= - tmpX(16)*tmpX(135)
      tmpX(19)=tmpX(143)*tmpX(64)*tmpX(14)
      tmpX(17)=tmpX(19) + tmpX(17)
      tmpX(17)=p4p5*tmpX(17)
      tmpX(16)= - tmpX(16)*tmpX(53)
      tmpX(14)=tmpX(156)*tmpX(48)*tmpX(14)
      tmpX(14)=tmpX(14) + tmpX(16)
      tmpX(14)=p3p5*tmpX(14)
      tmpX(14)=tmpX(17) + tmpX(14)
      tmpX(14)=Iz1z2*tmpX(14)
      tmpX(16)=tmpX(96)*tmpX(23)*tmpX(100)
      tmpX(12)=tmpX(14) + tmpX(12) + tmpX(16) + tmpX(15)
      tmpX(12)=Iz3z4*tmpX(12)
      tmpX(14)= - Iz1z2*tmpX(54)
      tmpX(14)=tmpX(14) - tmpX(30)
      tmpX(14)=tmpX(14)*tmpX(33)
      tmpX(15)=tmpX(58) + tmpX(18)
      tmpX(15)=tmpX(15)*den023m
      tmpX(15)=tmpX(15) + 1.D0/8.D0*tmpX(86)
      tmpX(16)=tmpX(15)*tmpX(135)
      tmpX(17)=1.D0/64.D0*tmpX(81)
      tmpX(19)= - tmpX(49)*tmpX(17)
      tmpX(13)= - 1.D0/4.D0*tmpX(13) - tmpX(18)
      tmpX(13)=den014m*tmpX(13)
      tmpX(13)=tmpX(19) + tmpX(13)
      tmpX(13)=tmpX(13)*tmpX(143)
      tmpX(13)=tmpX(13) + tmpX(16)
      tmpX(13)=p4p5*tmpX(13)
      tmpX(15)=tmpX(15)*tmpX(53)
      tmpX(16)= - tmpX(154)*tmpX(17)
      tmpX(17)= - 1.D0/4.D0*tmpX(60) - tmpX(69)
      tmpX(17)=den023m*tmpX(17)
      tmpX(16)=tmpX(16) + tmpX(17)
      tmpX(16)=tmpX(16)*tmpX(156)
      tmpX(15)=tmpX(16) + tmpX(15)
      tmpX(15)=p3p5*tmpX(15)
      tmpX(13)=tmpX(13) + tmpX(15)
      tmpX(13)=tmpX(45)*tmpX(139)*tmpX(13)
      tmpX(7)=tmpX(7) + tmpX(13) + tmpX(14) + tmpX(11) + tmpX(12)

      hardlam0 = tmpX(7)
      hardlam2 = tmpX(9)
      hardlam12 = tmpX(8)
      hardlamep = tmpX(10)



      if (alr.eq.0) then
         hard = hardlam0 + hardlam2*(lamem-lamep) + hardlamep*lamep + hardlam12*(1d0-lamem*lamep)
      elseif (alr.eq.1) then
         hard =  -4d0*hardlam2+2d0*hardlamep
      elseif (alr.eq.2) then
         hard =  2d0*hardlam0+4d0*hardlam12
      endif

      hard = hard*conhc*cfprime
      if (hard.ne.hard) print *,'Hard NAN!!!',s,spr,ph4,eta4,eta5


      return
      end
