#include "General.h"
#include "Process.h"
#include "Info.h"
using namespace SANC;

Process::Process(Info & _info):info(_info)
{
	idup.resize(4);
	idup[0]=-11;
	idup[1]=11;
    switch (info.Pid) {
    case 101:
	procName = "e+e- -> e-e+";
	arXiv = "Phys.Rev.D 98, 013001; arXiv:1801.00125";
	Mf1 = info.Mel;
	Mf2 = info.Mel;
	Mf3 = info.Mel;
	Mf4 = info.Mel;
	idup[2] = 11;
	idup[3] = -11;
	break;
    case 102:
	procName = "e+e- -> ZH";
	arXiv = "Phys.Rev.D 100, 073002; arXiv:1812.10965";
	Mf1 = info.Mel;
	Mf2 = info.Mel;
	Mf3 = info.MZ;
	Mf4 = info.MH;
	idup[2] = 23;
	idup[3] = 25;
	break;
    default:
	std::cerr<<"Error: Unknown process "<<info.Pid<<std::endl;
	exit(1);
    }
    kDimBorn = 2;
    kDimVirt = 2;
    kDimHard = 5;
    Mf1_2 = pow2(Mf1);
    Mf2_2 = pow2(Mf2);
    Mf3_2 = pow2(Mf3);
    Mf4_2 = pow2(Mf4);
    dm = (Mf1-Mf2)*(Mf1+Mf2)*(Mf4-Mf3)*(Mf4+Mf3);
    sm = Mf1_2+Mf2_2+Mf3_2+Mf4_2;
	event.particles.clear();
	for (auto const & id:  idup) {
		event.particles.push_back(Particle(id));
	}
	event.particles.push_back(Particle(22));

	// std::cout<<"-----------------------------------------------------"<<'\n';
	std::cout<<" Process: "<<procName<<'\n';
	std::cout<<" Citation: "<<arXiv<<'\n';
	std::cout<<" ===================================================="<<'\n';
}
