# ReneSANCe

ReneSANCe MC event generator

## Compilation
```shell
cmake <path_to_source> <other options>
cmake --build .
cmake --build . --target install
```
    By default -DCMAKE_INSTALL_PREFIX=<inst_prefix> is set to <path_to_source> directory.

## Run
```shell
source <inst_prefix>/bin/renesance-init.sh
renesance
```

## Directory structure
- Analysis: contains utilities to make histograms from ROOT files with generated events
    - make_plot_1d.cpp: 1d histograms for certain observables
    - make_plot_1d_alr.cpp: 1d distributions for LR-asymmetry
    - make_plot_2d.cpp: 2d histograms for observables
    - make_plot_2d_hard.cpp: 2d histograms in internal variables for debug purposes
    - CMakeLists.txt: internal CMake file
- build: folder to build sources
- cmake: CMake files to search external dependencies
- external: external packages
    - CLI11: command line parser for c++
    - libucl.tar.gz: universal configuration library parser 
    - LoopTools.tar.gz: package for evaluation of scalar and tensor one-loop integrals
- Generator: MC event generator files
    - doc: supplementary documentation
        - Kinematics.pdf: brief description of kinematics
    - examples:
        - 101: input settings and sample output for Bhabha scattering
        - 102: input settings and sample output for Higgs-strahlung
    - include: header files of generator
        - DataSaver.h: class for saving events to disk
        - General.h:  often used helper functions
        - Info.h: stores main parameters
        - Integrand.h: integrand functions
        - LHEF.h:  implements Les Houches Events files writer
        - ModulesInterface.h: initializes variables for fortran modules using Info class
        - OF_LHEF.h: implementation of OutputFormat class for LHEF
        - OutputFormat.h: abstract class to provide data for different output formats
        - Process.h: stores process dependent information
        - ReneSANCe_ee.h: main class of generator
        - Settings.h: abstract class for operations with input settings
        - SettingsUCL.h: implementation of Settings class based on libUCL library
        - Time.h: performance measurements
        - Vec4.h: class for Lorenz 4-vectors
    - modules: fortran SANC modules for different components of differential cross section
    - SANC: SANC library for specific loop integrals and universal h.o. corrections
    - schema: input parameters with default values used internally in generator (generally should not be modified by user)
    - src: implementation of classes listed in include folder
    - CMakeLists.txt: internal CMake file
    - renesance-init.sh: script to set up environment for the ReneSANCe MC generator
- CMakeLists.txt: main CMake file of project
- README.md: short program description

## Documentation
Read https://arxiv.org/abs/2001.10755 for details.
